/*
 * GPJColor.java v1.4
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class GPJColor
{
    
    public int m_red;
    public int m_green;
    public int m_blue;
    public int m_RGB;

    public GPJColor()
    {
        m_red   = 0;
        m_green = 0;
        m_blue  = 0;

        m_RGB = 0;
    }
    
    public GPJColor(int red,
                    int green,
                    int blue)
    {       
        set(red,
                green,
                blue);
    }
    
    public void calcRGB()
    {
        int red     = m_red;
        int green   = m_green;
        int blue    = m_blue;

        if (red < 0)
            red = 0;
        else if (red > 255)
            red = 255;

        if (green < 0)
            green = 0;
        else if (green > 255)
            green = 255;

        if (blue < 0)
            blue = 0;
        else if (blue > 255)
            blue = 255;

        m_RGB = (m_red << 16) | (m_green << 8) | m_blue;
    }

    public void calcRedGreenBlue()
    {
        m_red   = (m_RGB & 0x00ff0000) >> 16;
        m_green = (m_RGB & 0x0000ff00) >> 8;
        m_blue  = m_RGB & 0x000000ff;
    }

    public void set(int red,
                    int green,
                    int blue)
    {
        m_red   = red;
        m_green = green;
        m_blue  = blue;

        calcRGB();
    }

    public static void copyColor(GPJColor src,
                                 GPJColor dst)
    {
        dst.m_red   = src.m_red;
        dst.m_green = src.m_green;
        dst.m_blue  = src.m_blue;

        dst.m_RGB = src.m_RGB;
    }

    public static void subColors(GPJColor src1,
                                 GPJColor src2,
                                 GPJColor dst)
    {
        dst.m_red   = src1.m_red - src2.m_red;
        dst.m_green = src1.m_green - src2.m_green;
        dst.m_blue  = src1.m_blue - src2.m_blue;
    }

    public static void multiplyColors(GPJColor src1,
                                      GPJColor src2,
                                      GPJColor dst)
    {
        dst.m_red   = src1.m_red * src2.m_red / 255;
        dst.m_green = src1.m_green * src2.m_green / 255;
        dst.m_blue  = src1.m_blue * src2.m_blue / 255;
    }
    
    public static void interpolatesColors(GPJColor  src1,
                                          GPJColor  src2,
                                          GPJColor  dst,
                                          float     rate)
    {
        float r1;
        float g1;
        float b1;
        float r2;
        float g2;
        float b2;
        
        r1 = (float)src1.m_red;
        g1 = (float)src1.m_green;
        b1 = (float)src1.m_blue;
        r2 = (float)src2.m_red;
        g2 = (float)src2.m_green;
        b2 = (float)src2.m_blue;

        dst.m_red   = (int)(r1 + (r2 - r1) * rate);
        dst.m_green = (int)(g1 + (g2 - g1) * rate);
        dst.m_blue  = (int)(b1 + (b2 - b1) * rate);
    }

    public static void exchangeColors(GPJColor color1,
                                      GPJColor color2)
    {
        int red;
        int green;
        int blue;
        int RGB;

        red             = color1.m_red;
        green           = color1.m_green;
        blue            = color1.m_blue;
        RGB             = color1.m_RGB;
        color1.m_red    = color2.m_red;
        color1.m_green  = color2.m_green;
        color1.m_blue   = color2.m_blue;
        color1.m_RGB    = color2.m_RGB;
        color2.m_red    = red;
        color2.m_green  = green;
        color2.m_blue   = blue;
        color2.m_RGB    = RGB;
    }

    @Override
    public String toString()
    {
        return super.toString() + "  red: " + m_red + "  green: " + m_green + "  blue: " + m_blue;
    }
    
}
