/*
 * GPJLight.java v1.2
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class GPJLight
{
    
    public static final int TYPE_AMBIENT        = 1;
    public static final int TYPE_DIRECTIONAL    = 2;
    public static final int TYPE_POINT          = 4;
    public static final int TYPE_SPOT           = 8;
    
    public int              m_type;
    public GPJVector3D      m_pos;
    public GPJVector3D      m_angles;
    public GPJColor         m_color;

    protected GPJVector3D   m_vectorRot;
    protected GPJMatrix     m_matrixRot;

    public GPJLight()
    {
        m_type = TYPE_AMBIENT;

        m_pos       = new GPJVector3D();
        m_angles    = new GPJVector3D();

        m_color = new GPJColor();

        m_vectorRot = new GPJVector3D();
        m_matrixRot = new GPJMatrix();
    }

    public GPJLight(GPJColor color)
    {
        this();

        GPJColor.copyColor(color,
                m_color);
    }

    public GPJLight(float       angleX,
                    float       angleY,
                    GPJColor    color)
    {
        this(color);

        m_type = TYPE_DIRECTIONAL;

        m_angles.set(angleX,
                angleY,
                0.0f);

        updateAngulation();
    }

    public GPJLight(float       posX,
                    float       posY,
                    float       posZ,
                    GPJColor    color)
    {
        this(color);

        m_type = TYPE_POINT;

        m_pos.set(posX,
                posY,
                posZ);
    }

    public GPJLight(float       posX,
                    float       posY,
                    float       posZ,
                    float       angleX,
                    float       angleY,
                    GPJColor    color)
    {
        this(color);

        m_type = TYPE_SPOT;

        m_pos.set(posX,
                posY,
                posZ);
        m_angles.set(angleX,
                angleY,
                0.0f);

        updateAngulation();
    }

    public void updateAngulation()
    {
        m_matrixRot.setIdentity();
        m_matrixRot.rotate(m_angles.m_x,
                m_angles.m_y,
                m_angles.m_z);

        GPJVector3D.multiplyVectorMatrix(new GPJVector3D(0.0f, 0.0f, 1.0f),
                m_matrixRot,
                m_vectorRot);
    }

}
