/*
 * GPJMatrix.java v1.2
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class GPJMatrix
{

    public float[] m_matrix;
    
    public GPJMatrix()
    {
        m_matrix = new float[9];
    }

    public static void copyMatrix(GPJMatrix src,
                                  GPJMatrix dst)
    {
        System.arraycopy(src.m_matrix,
                0,
                dst.m_matrix,
                0,
                9);
    }
    
    public static void multiplyMatrices(GPJMatrix src1,
                                        GPJMatrix src2,
                                        GPJMatrix dst)
    {
        int x;
        int y;
        int offsetY;

        for (x = 0; x < 3; x ++)
        {
            for (y = 0, offsetY = 0; y < 3; y ++, offsetY += 3)
                dst.m_matrix[x + offsetY] = src1.m_matrix[x] * src2.m_matrix[offsetY] +
                        src1.m_matrix[x + 3] * src2.m_matrix[offsetY + 1] +
                        src1.m_matrix[x + 6] * src2.m_matrix[offsetY + 2];
        }
    }
    
    public static void invertMatrix(GPJMatrix src,
                                    GPJMatrix dst)
    {
        int i;
        int offsetX = 0;
        int offsetY = 0;
        
        for (i = 0; i < 9; i ++, offsetY += 3)
        {
            if (offsetY > 6)
            {
                offsetX ++;
                offsetY = 0;
            }
            dst.m_matrix[i] = src.m_matrix[offsetX + offsetY];
        }
    }
    
    public void setIdentity()
    {
        m_matrix[0] = 1;
        m_matrix[3] = 0;
        m_matrix[6] = 0;
        m_matrix[1] = 0;
        m_matrix[4] = 1;
        m_matrix[7] = 0;
        m_matrix[2] = 0;
        m_matrix[5] = 0;
        m_matrix[8] = 1;
    }

    public void rotate(float angleX,
                       float angleY,
                       float angleZ)
    {
        float       sAngleX;
        float       cAngleX;
        float       sAngleY;
        float       cAngleY;
        float       sAngleZ;
        float       cAngleZ;
        GPJMatrix   matrixTemp1;
        GPJMatrix   matrixTemp2;
        GPJMatrix   matrixTempX;
        GPJMatrix   matrixTempY;
        GPJMatrix   matrixTempZ;

        angleX = (float)Math.toRadians(angleX);
        angleY = (float)Math.toRadians(angleY);
        angleZ = (float)Math.toRadians(angleZ);

        sAngleX = (float)Math.sin(angleX);
        cAngleX = (float)Math.cos(angleX);
        sAngleY = (float)Math.sin(angleY);
        cAngleY = (float)Math.cos(angleY);
        sAngleZ = (float)Math.sin(angleZ);
        cAngleZ = (float)Math.cos(angleZ);

        matrixTemp1 = new GPJMatrix();
        matrixTemp2 = new GPJMatrix();
        matrixTempX = new GPJMatrix();
        matrixTempY = new GPJMatrix();
        matrixTempZ = new GPJMatrix();
        
        matrixTempX.setIdentity();
        matrixTempX.m_matrix[4] = cAngleX;
        matrixTempX.m_matrix[5] = sAngleX;
        matrixTempX.m_matrix[7] = -sAngleX;
        matrixTempX.m_matrix[8] = cAngleX;

        matrixTempY.setIdentity();
        matrixTempY.m_matrix[0] = cAngleY;
        matrixTempY.m_matrix[2] = -sAngleY;
        matrixTempY.m_matrix[6] = sAngleY;
        matrixTempY.m_matrix[8] = cAngleY;

        matrixTempZ.setIdentity();
        matrixTempZ.m_matrix[0] = cAngleZ;
        matrixTempZ.m_matrix[1] = sAngleZ;
        matrixTempZ.m_matrix[3] = -sAngleZ;
        matrixTempZ.m_matrix[4] = cAngleZ;

        multiplyMatrices(this,
                matrixTempZ,
                matrixTemp1);
        multiplyMatrices(matrixTemp1,
                matrixTempY,
                matrixTemp2);
        multiplyMatrices(matrixTemp2,
                matrixTempX,
                this);
    }
    
}
