/*
 * GPJPlane.java v1.1
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class GPJPlane
{

    public float        m_a;
    public float        m_b;
    public float        m_c;
    public float        m_d;
    public GPJVector3D  m_normal;

    public GPJPlane()
    {
        m_normal = new GPJVector3D();
    }

    public GPJPlane(float a,
                    float b,
                    float c,
                    float d)
    {
        this();

        set(a,
                b,
                c,
                d);
    }

    public void set(float a,
                    float b,
                    float c,
                    float d)
    {
        m_a = a;
        m_b = b;
        m_c = c;
        m_d = d;

        m_normal.set(m_a,
                m_b,
                m_c);
    }

    public void set(GPJVector3D vector1,
                    GPJVector3D vector2,
                    GPJVector3D vector3)
    {
        GPJVector3D edge1;
        GPJVector3D edge2;
   
        edge1   = new GPJVector3D();
        edge2   = new GPJVector3D();
      
        GPJVector3D.subVectors(vector3,
                vector1,
                edge1);
        GPJVector3D.subVectors(vector2,
                vector1,
                edge2);

        GPJVector3D.crossProductVectors(edge1,
                edge2,
                m_normal);
        GPJVector3D.normalizeVector(m_normal,
                m_normal);

        m_a = m_normal.m_x;
        m_b = m_normal.m_y;
        m_c = m_normal.m_z;
        m_d = -GPJVector3D.dotProductVectors(m_normal,
                vector1);
    }

    public static void copyPlane(GPJPlane src,
                                 GPJPlane dst)
    {
        dst.m_a = src.m_a;
        dst.m_b = src.m_b;
        dst.m_c = src.m_c;
        dst.m_d = src.m_d;

        GPJVector3D.copyVector(src.m_normal,
                dst.m_normal);
    }

    public float getDistanceFromVector(GPJVector3D vector)
    {
        return vector.m_x * m_a + vector.m_y * m_b + vector.m_z * m_c + m_d;
    }

    @Override
    public String toString()
    {
        return super.toString() + "  a: " + m_a + "  b: " + m_b + "  c: " + m_c + "  d: " + m_d;
    }

}
