/*
 * GPJPoint3D.java v1.3
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class GPJPoint3D extends GPJVector3D
{

    public boolean      m_trasformTo2D;
    public int          m_xOnScreen;
    public int          m_yOnScreen;
    public GPJColor     m_color;
    public GPJVector2D  m_uv;
    public GPJVector3D  m_normal;

    protected GPJColor  m_colorProcessed;

    private boolean     m_onScreen;
    
    public GPJPoint3D()
    {
        super();

        m_color             = new GPJColor();
        m_colorProcessed    = new GPJColor();

        m_uv = new GPJVector2D();

        m_normal = new GPJVector3D();

        reset();
    }
    
    public GPJPoint3D(float x,
                      float y,
                      float z)
    {
        this();

        this.set(x,
                y,
                z);

        reset();
    }
    
    public GPJPoint3D(float     x,
                      float     y,
                      float     z,
                      GPJColor  color)
    {
        this(x,
                y,
                z);

        GPJColor.copyColor(color,
                m_color);

        reset();
    }

    public GPJPoint3D(float     x,
                      float     y,
                      float     z,
                      GPJColor  color,
                      float     u,
                      float     v)
    {
        this(x,
                y,
                z,
                color);

        m_uv.set(u,
                v);

        reset();
    }

    public boolean isVisible()
    {
        return m_onScreen;
    }

    protected void reset()
    {
        m_trasformTo2D = false;

        m_onScreen  = false;
        m_xOnScreen = -1;
        m_yOnScreen = -1;

        m_normal.set(0.0f,
                0.0f,
                0.0f);
    }

    protected void trasformTo2D(int     screenCenterX,
                                int     screenCenterY,
                                float   screenDistX,
                                float   screenDistY)
    {
        if (m_trasformTo2D)
        {
            if (Math.abs(m_z) > GPJEngine3D.TOLLERANCE)
            {
                m_onScreen  = true;
                m_xOnScreen = screenCenterX + (int)(screenDistX * m_x / m_z);
                m_yOnScreen = screenCenterY + (int)(screenDistY * m_y / m_z);
            }
        }
    }
    
}
