/*
 * GPJShowRoom.java v1.5
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Modulo compatibile con JVM v1.1 o superiore
 */

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class GPJShowRoom extends JApplet implements Runnable, MouseListener, MouseMotionListener, KeyListener
{

    private int                 m_fps;
    private int                 m_mouseStartPosX;
    private int                 m_mouseStartPosY;
    private int                 m_screenWidth;
    private int                 m_screenHeight;
    private int                 m_lightType;
    private long                m_startTime;
    private long                m_endTime;
    private float               m_lightAngleY;
    private MemoryImageSource   m_memoryImageSource;
    private DirectColorModel    m_directColorModel;
    private Image               m_image;
    private Thread              m_thread;
    private JComponent          m_contentPane;

    private GPJEngine3D         m_engine3D;
    private GPJCamera           m_camera;
    private GPJPoint3D          m_vertex1;
    private GPJPoint3D          m_vertex2;
    private GPJPoint3D          m_vertex3;
    private GPJPoint3D          m_vertex4;
    private GPJPoint3D          m_vertex5;
    private GPJPoint3D          m_vertex6;
    private GPJPoint3D          m_vertex7;
    private GPJPoint3D          m_vertex8;
    private GPJPoint3D          m_vertex9;
    private GPJPoint3D          m_vertex10;
    private GPJLine             m_line;
    private GPJTexture          m_texture0;
    private GPJTriangle         m_triangle1;
    private GPJTriangle         m_triangle2;
    private GPJTriangle         m_triangle3;
    private GPJTriangle         m_triangle4;
    private GPJTriangle         m_triangle5;
    private GPJTriangle         m_triangle6;
    private GPJTriangle         m_triangle7;
    private GPJTriangle         m_triangle8;
    private GPJTriangle         m_triangle9;
    private GPJTriangle         m_triangle10;
    private GPJTriangle         m_triangle11;
    private GPJTriangle         m_triangle12;
    private GPJLight            m_lightAmbient;
    private GPJLight            m_lightDirectional;
    private GPJLight            m_lightPoint;
    private GPJLight            m_lightSpot;
    private Vector              m_listPoints;

    @Override
    public void init()
    {
        float x;
        float z;

        setFocusable(true);

        m_lightType     = GPJLight.TYPE_DIRECTIONAL;
        m_lightAngleY   = 0.0f;

        m_engine3D = new GPJEngine3D(new GPJColor(0, 140, 190),
                GPJEngine3D.GOURAUD_SHADING);
        
        m_camera = new GPJCamera(0.0f,
                0.0f,
                0.0f,
                180.0f,
                180.0f,
                180.0f);

        m_vertex1 = new GPJPoint3D(-50.0f,
                -50.0f,
                150.0f,
                new GPJColor(255, 255, 255),
                0.0f,
                0.0f);
        m_vertex2 = new GPJPoint3D(50.0f,
                -50.0f,
                150.0f,
                new GPJColor(255, 255, 255),
                1.0f,
                0.0f);
        m_vertex3 = new GPJPoint3D(50.0f,
                50.0f,
                150.0f,
                new GPJColor(255, 255, 255),
                1.0f,
                1.0f);
        m_vertex4 = new GPJPoint3D(-50.0f,
                50.0f,
                150.0f,
                new GPJColor(255, 255, 255),
                0.0f,
                1.0f);
        m_vertex5 = new GPJPoint3D(-50.0f,
                -50.0f,
                250.0f,
                new GPJColor(0, 0, 255));
        m_vertex6 = new GPJPoint3D(50.0f,
                -50.0f,
                250.0f,
                new GPJColor(0, 0, 255));
        m_vertex7 = new GPJPoint3D(50.0f,
                50.0f,
                250.0f,
                new GPJColor(255, 0, 0));
        m_vertex8 = new GPJPoint3D(-50.0f,
                50.0f,
                250.0f,
                new GPJColor(255, 0, 0));
        m_vertex9 = new GPJPoint3D(0.0f,
                0.0f,
                100.0f,
                new GPJColor(255, 255, 0));
        m_vertex10 = new GPJPoint3D(0.0f,
                0.0f,
                300.0f,
                new GPJColor(255, 0, 0));

        m_line = new GPJLine(8,
                9);

        m_texture0 = new GPJTexture(getClass().getResource("texture0.png"),
                getClass().getResource("texture0_alpha.png"));

        m_triangle1 = new GPJTriangle(m_texture0,
                0,
                1,
                3);
        m_triangle2 = new GPJTriangle(m_texture0,
                2,
                3,
                1);
        m_triangle3 = new GPJTriangle(null,
                0,
                4,
                1);
        m_triangle4 = new GPJTriangle(null,
                5,
                1,
                4);
        m_triangle5 = new GPJTriangle(null,
                1,
                5,
                6);
        m_triangle6 = new GPJTriangle(null,
                1,
                6,
                2);
        m_triangle7 = new GPJTriangle(null,
                4,
                0,
                3);
        m_triangle8 = new GPJTriangle(null,
                4,
                3,
                7);
        m_triangle9 = new GPJTriangle(null,
                5,
                4,
                7);
        m_triangle9.m_culling = false;
        m_triangle10 = new GPJTriangle(null,
                5,
                7,
                6);
        m_triangle10.m_culling = false;
        m_triangle11 = new GPJTriangle(null,
                3,
                2,
                6);
        m_triangle12 = new GPJTriangle(null,
                3,
                6,
                7);

        m_lightAmbient      = new GPJLight(new GPJColor(128, 128, 160));
        m_lightDirectional  = new GPJLight(180.0f,
                m_lightAngleY,
                new GPJColor(255, 255, 190));
        m_lightPoint        = new GPJLight(0.0f,
                0.0f,
                0.0f,
                new GPJColor(255, 255, 190));
        m_lightSpot         = new GPJLight(0.0f,
                0.0f,
                0.0f,
                180.0f,
                0.0f,
                new GPJColor(255, 255, 190));
       
        m_engine3D.setCamera(m_camera);

        m_engine3D.addLight(m_lightAmbient);
        m_engine3D.addLight(m_lightDirectional);

        m_listPoints = new Vector();

        for (x = -200.0f; x <= 200.0f; x += 10.0f)
            for (z = 0.0f; z <= 600.0f; z += 10.0f)
                m_listPoints.addElement(new GPJPoint3D(x, 50.0f, z, new GPJColor(255, 255, 255)));

        for (x = -200.0f; x <= 200.0f; x += 10.0f)
            for (z = 0.0f; z <= 600.0f; z += 10.0f)
                m_listPoints.addElement(new GPJPoint3D(x, -50.0f, z, new GPJColor(255, 255, 255)));

        m_contentPane = new JPanel()
        {
            @Override
            public void paintComponent(Graphics g)
            {
                super.paintComponent(g);

                if (m_image != null)
                {
                    g.drawImage(m_image,
                            0,
                            0,
                            this);
                    g.setColor(Color.white);
                    g.drawString("fps: " + m_fps,
                            5,
                            15);
                    g.drawString("px: " + m_camera.m_pos.m_x + "  py: " + m_camera.m_pos.m_y + "  pz: " + m_camera.m_pos.m_z,
                            5,
                            29);
                    g.drawString("ax: " + m_camera.m_angles.m_x + "  ay: " + m_camera.m_angles.m_y + "  az: " + m_camera.m_angles.m_z,
                            5,
                            43);

                    if (m_engine3D.m_shading == GPJEngine3D.FLAT_SHADING)
                        g.drawString("Flat shading",
                            5,
                            57);
                    else if (m_engine3D.m_shading == GPJEngine3D.GOURAUD_SHADING)
                        g.drawString("Gouraud shading",
                            5,
                            57);

                    if (m_lightType == 0)
                        g.drawString("Only ambient light",
                            5,
                            71);
                    else if (m_lightType == GPJLight.TYPE_DIRECTIONAL)
                        g.drawString("Directional light",
                            5,
                            71);
                    else if (m_lightType == GPJLight.TYPE_POINT)
                        g.drawString("Point light",
                            5,
                            71);
                    else if (m_lightType == GPJLight.TYPE_SPOT)
                        g.drawString("Spot light",
                            5,
                            71);
                }
            }
        };
        m_contentPane.setBackground(Color.white);
        m_contentPane.addMouseListener(this);
        m_contentPane.addMouseMotionListener(this);
        m_contentPane.addKeyListener(this);
        setContentPane(m_contentPane);

        addMouseListener(this);
        addMouseMotionListener(this);
        addKeyListener(this);
    }

    @Override
    public void start()
    {
        m_screenWidth   = m_contentPane.getSize().width;
        m_screenHeight  = m_contentPane.getSize().height;
        
        initScreen(m_screenWidth,
                m_screenHeight);
    }

    @Override
    public void run()
    {
        int         i;
        int         frames = 0;
        /*int newWidth;
        int newHeight;*/
        GPJPoint3D  point;

        m_fps = 0;

        m_startTime = m_endTime = System.currentTimeMillis();

        try
        {
            while (m_thread == Thread.currentThread())
            {
                frames ++;

                if (m_lightType == GPJLight.TYPE_DIRECTIONAL)
                {
                    m_lightAngleY += 1.0f;

                    m_lightDirectional.m_angles.m_y = m_lightAngleY;
                    m_lightDirectional.updateAngulation();

                    m_vertex9.set(m_lightDirectional.m_vectorRot.m_x * 150.0f,
                            m_lightDirectional.m_vectorRot.m_y * 150.0f,
                            m_lightDirectional.m_vectorRot.m_z * 150.0f + 200.0f);

                    m_vertex10.set(m_lightDirectional.m_vectorRot.m_x * 130.0f,
                            m_lightDirectional.m_vectorRot.m_y * 130.0f,
                            m_lightDirectional.m_vectorRot.m_z * 130.0f + 200.0f);
                }
                else if (m_lightType == GPJLight.TYPE_POINT)
                {
                    m_lightAngleY += 1.0f;

                    m_lightPoint.m_pos.m_x = (float)Math.sin((float)Math.toRadians(m_lightAngleY + 180.0f)) * 140.0f;
                    m_lightPoint.m_pos.m_y = 0.0f;
                    m_lightPoint.m_pos.m_z = (float)Math.cos((float)Math.toRadians(m_lightAngleY + 180.0f)) * 140.0f + 200.0f;

                    m_vertex9.set(m_lightPoint.m_pos.m_x,
                            m_lightPoint.m_pos.m_y - 10.0f,
                            m_lightPoint.m_pos.m_z);

                    m_vertex10.set(m_lightPoint.m_pos.m_x,
                            m_lightPoint.m_pos.m_y + 10.0f,
                            m_lightPoint.m_pos.m_z);
                }
                else if (m_lightType == GPJLight.TYPE_SPOT)
                {
                    m_lightAngleY += 1.0f;

                    m_lightSpot.m_pos.m_x = (float)Math.sin((float)Math.toRadians(m_lightAngleY + 180.0f)) * 140.0f;
                    m_lightSpot.m_pos.m_y = 0.0f;
                    m_lightSpot.m_pos.m_z = (float)Math.cos((float)Math.toRadians(m_lightAngleY + 180.0f)) * 140.0f + 200.0f;

                    m_vertex9.set(m_lightSpot.m_pos.m_x,
                            m_lightSpot.m_pos.m_y,
                            m_lightSpot.m_pos.m_z);

                    m_vertex10.set(m_lightSpot.m_pos.m_x + m_lightSpot.m_vectorRot.m_x * -20,
                            m_lightSpot.m_pos.m_y + m_lightSpot.m_vectorRot.m_y * -20,
                            m_lightSpot.m_pos.m_z + m_lightSpot.m_vectorRot.m_z * -20);
                }

                /*newWidth    = getSize().width;
                newHeight   = getSize().height;

                if (newWidth != m_screenWidth ||
                        newHeight != m_screenHeight)
                {
                    initScreen(newWidth,
                            newHeight);

                    m_screenWidth   = newWidth;
                    m_screenHeight  = newHeight;
                }*/

                m_engine3D.init(true,
                        true);

                m_engine3D.addVertex(m_vertex1);
                m_engine3D.addVertex(m_vertex2);
                m_engine3D.addVertex(m_vertex3);
                m_engine3D.addVertex(m_vertex4);
                m_engine3D.addVertex(m_vertex5);
                m_engine3D.addVertex(m_vertex6);
                m_engine3D.addVertex(m_vertex7);
                m_engine3D.addVertex(m_vertex8);
                m_engine3D.addVertex(m_vertex9);
                m_engine3D.addVertex(m_vertex10);

                m_engine3D.addLine(m_line);

                m_engine3D.addTriangle(m_triangle3);
                m_engine3D.addTriangle(m_triangle4);
                m_engine3D.addTriangle(m_triangle5);
                m_engine3D.addTriangle(m_triangle6);
                m_engine3D.addTriangle(m_triangle7);
                m_engine3D.addTriangle(m_triangle8);
                m_engine3D.addTriangle(m_triangle9);
                m_engine3D.addTriangle(m_triangle10);
                m_engine3D.addTriangle(m_triangle11);
                m_engine3D.addTriangle(m_triangle12);

                for (i = 0; i < m_listPoints.size(); i ++)
                {
                    point = (GPJPoint3D)m_listPoints.elementAt(i);
                    if (point != null)
                        m_engine3D.addPoint(point);
                }

                m_engine3D.render();

                m_engine3D.init(false,
                        false);

                m_engine3D.addVertex(m_vertex1);
                m_engine3D.addVertex(m_vertex2);
                m_engine3D.addVertex(m_vertex3);
                m_engine3D.addVertex(m_vertex4);

                m_engine3D.addTriangle(m_triangle1);
                m_engine3D.addTriangle(m_triangle2);

                m_engine3D.render();

                m_memoryImageSource.newPixels();

                m_contentPane.repaint();
                
                Thread.sleep(1);

                m_endTime = System.currentTimeMillis();
                if (m_endTime - m_startTime >= 1000)
                {
                    m_fps   = frames;
                    frames  = 0;

                    m_startTime = m_endTime;
                }
            }
        }
        catch (Exception e)
        {
        }
    }

    @Override
    public void stop()
    {
        m_thread = null;

        System.gc();
    }

    @Override
    public void destroy()
    {
        m_thread = null;

        freeAllResources();

        System.gc();
    }

    @Override
    public void mousePressed(MouseEvent e)
    {
        if (e.getModifiers() == MouseEvent.BUTTON1_MASK)
        {
            m_mouseStartPosX = e.getX();
            m_mouseStartPosY = e.getY();
        }
    }

    @Override
    public void mouseReleased(MouseEvent e)
    {
    }

    @Override
    public void mouseEntered(MouseEvent e)
    {
    }

    @Override
    public void mouseExited(MouseEvent e)
    {
    }

    @Override
    public void mouseClicked(MouseEvent e)
    {
    }

    @Override
    public void mouseMoved(MouseEvent e)
    {
    }

    @Override
    public void mouseDragged(MouseEvent e)
    {
        int diffX;
        int diffY;
        int mouseNewPosX;
        int mouseNewPosY;

        if (e.getModifiers() == MouseEvent.BUTTON1_MASK)
        {
            mouseNewPosX = e.getX();
            mouseNewPosY = e.getY();

            diffX = m_mouseStartPosY - mouseNewPosY;
            diffY = m_mouseStartPosX - mouseNewPosX;

            m_mouseStartPosX = mouseNewPosX;
            m_mouseStartPosY = mouseNewPosY;

            m_camera.rotate(90.0f / (float)m_screenWidth * (float)diffX,
                    90.0f / (float)m_screenHeight * (float)diffY,
                    0.0f);
        }
    }

    @Override
    public void keyTyped(KeyEvent e)
    {
    }

    @Override
    public void keyPressed(KeyEvent e)
    {
        int key = e.getKeyCode();

        if (key == KeyEvent.VK_UP)
            m_camera.move(GPJCamera.TYPE_MOVE_FORWARD,
                    5.0f);
        else if (key == KeyEvent.VK_DOWN)
            m_camera.move(GPJCamera.TYPE_MOVE_BACK,
                    5.0f);
        else if (key == KeyEvent.VK_LEFT)
            m_camera.move(GPJCamera.TYPE_MOVE_LEFT,
                    5.0f);
        else if (key == KeyEvent.VK_RIGHT)
            m_camera.move(GPJCamera.TYPE_MOVE_RIGHT,
                    5.0f);
    }

    @Override
    public void keyReleased(KeyEvent e)
    {
        int key = e.getKeyCode();

        if (key == KeyEvent.VK_F1)
        {
            if (m_engine3D.m_shading == GPJEngine3D.FLAT_SHADING)
                m_engine3D.m_shading = GPJEngine3D.GOURAUD_SHADING;
            else if (m_engine3D.m_shading == GPJEngine3D.GOURAUD_SHADING)
                m_engine3D.m_shading = GPJEngine3D.FLAT_SHADING;
        }
        else if (key == KeyEvent.VK_F2)
        {
            if (m_lightType == 0)
            {
                m_lightType = GPJLight.TYPE_DIRECTIONAL;

                m_engine3D.addLight(m_lightDirectional);

                m_engine3D.addLine(m_line);
            }
            else if (m_lightType == GPJLight.TYPE_DIRECTIONAL)
            {
                m_lightType = GPJLight.TYPE_POINT;

                m_engine3D.addLight(m_lightPoint);
            }
            else if (m_lightType == GPJLight.TYPE_POINT)
            {
                m_lightType = GPJLight.TYPE_SPOT;

                m_engine3D.addLight(m_lightSpot);
            }
            else if (m_lightType == GPJLight.TYPE_SPOT)
            {
                m_lightType = 0;
                
                m_engine3D.addLight(null);
                
                m_engine3D.removeLine(m_line);
            }
        }
    }

    private void initScreen(int screenWidth,
                            int screenHeight)
    {      
        m_thread = null;
        
        freeAllResources();

        System.gc();

        if (m_camera.setScreen(60, screenWidth, screenHeight, 5.0f, 300.0f))
        {
            m_directColorModel  = new DirectColorModel(32,
                    0x00FF0000,
                    0x0000FF00,
                    0x000000FF,
                    0);
            m_memoryImageSource = new MemoryImageSource(screenWidth,
                    screenHeight,
                    m_directColorModel,
                    m_camera.m_screenBuffer,
                    0,
                    screenWidth);

            m_memoryImageSource.setAnimated(true);
            m_memoryImageSource.setFullBufferUpdates(true);

            m_image = createImage(m_memoryImageSource);

            m_thread = new Thread(this);
            
            m_thread.start();
        }
    }
    
    private void freeAllResources()
    {
        m_directColorModel  = null;
        m_memoryImageSource = null;
        
        m_image = null;
    }

    @Override
    public String getAppletInfo()
    {
        return "Title: GPJShowRoom v1.5\nAuthor: Giuseppe Perniola";
    }
    
}