/*
 * GPJTexture.java v1.1
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.net.URL;

public class GPJTexture
{

    public int[]    m_textureBuffer;
    public float[]  m_alphaBuffer;

    private int     m_width;
    private int     m_height;

    public GPJTexture(URL textureURL,
                      URL alphaURL)
    {
        int             i;
        int[]           alphaBuffer;
        ImageIcon       textureIcon;
        Image           textureImage;
        BufferedImage   textureBufferedImage;
        Graphics2D      graphics;

        m_textureBuffer = null;
        m_alphaBuffer   = null;

        try
        {
            textureIcon = new ImageIcon(textureURL);

            textureImage = textureIcon.getImage();

            m_width     = textureIcon.getIconWidth();
            m_height    = textureIcon.getIconHeight();

            textureBufferedImage = new BufferedImage(m_width,
                    m_height,
                    BufferedImage.TYPE_INT_RGB);

            graphics = textureBufferedImage.createGraphics();
            graphics.drawImage(textureImage,
                    0,
                    0,
                    null);

            m_textureBuffer = new int[m_width * m_height];

            textureBufferedImage.getRGB(0,
                    0,
                    m_width,
                    m_height,
                    m_textureBuffer,
                    0,
                    m_width);

            if (alphaURL != null)
            {
                textureIcon = new ImageIcon(alphaURL);

                textureImage = textureIcon.getImage();

                textureBufferedImage = new BufferedImage(m_width,
                        m_height,
                        BufferedImage.TYPE_BYTE_GRAY);

                graphics = textureBufferedImage.createGraphics();
                graphics.drawImage(textureImage,
                        0,
                        0,
                        null);

                alphaBuffer     = new int[m_width * m_height];
                m_alphaBuffer   = new float[m_width * m_height];

                textureBufferedImage.getRGB(0,
                        0,
                        m_width,
                        m_height,
                        alphaBuffer,
                        0,
                        m_width);

                for (i = 0; i < alphaBuffer.length; i ++)
                    m_alphaBuffer[i] = (float)(alphaBuffer[i] & 0xff) / 255.0f;
            }
        }
        catch (Exception e)
        {
            System.out.println(e.toString());
        }
    }

    public int getWidth()
    {
        return m_width;
    }

    public int getHeight()
    {
        return m_height;
    }

}
