/*
 * GPJTriangle.java v1.4
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class GPJTriangle
{
    
    protected static final int  NUM_VERTEXES            = 3;
    protected static final int  NUM_VERTEXES_CLIPPED    = NUM_VERTEXES * 2;

    public boolean              m_culling;

    protected int               m_numVertexesClipped;
    protected int[]             m_listNumVertexes;
    protected GPJTexture        m_texture;
    protected GPJPlane          m_plane;
    protected GPJPoint3D[]      m_listVertexesProcessed;
    protected GPJPoint3D[]      m_listVertexesClipped;
    
    public GPJTriangle(GPJTexture   texture,
                       int          vertex1,
                       int          vertex2,
                       int          vertex3)
    {
        m_culling = true;

        m_listNumVertexes       = new int[NUM_VERTEXES];
        m_listNumVertexes[0]    = vertex1;
        m_listNumVertexes[1]    = vertex2;
        m_listNumVertexes[2]    = vertex3;

        m_texture = texture;

        m_plane = new GPJPlane();

        m_listVertexesProcessed = new GPJPoint3D[NUM_VERTEXES];
        
        m_numVertexesClipped    = 0;
        m_listVertexesClipped   = new GPJPoint3D[NUM_VERTEXES_CLIPPED];
    }

    public boolean isVisible()
    {
        boolean     ok = false;
        int         i;
        float       c;
        GPJVector3D vertex1;
        GPJVector3D vertex2;
        GPJVector3D vertex3;

        for (i = 0; i < NUM_VERTEXES; i ++)
            if (m_listVertexesProcessed[i].m_trasformTo2D)
            {
                ok = true;

                break;
            }

        if (ok &&
                m_culling)
        {
            vertex1 = m_listVertexesProcessed[0];
            vertex2 = m_listVertexesProcessed[1];
            vertex3 = m_listVertexesProcessed[2];

            c = vertex3.m_x * ((vertex1.m_z * vertex2.m_y) - (vertex1.m_y * vertex2.m_z)) +
                    vertex3.m_y * ((vertex1.m_x * vertex2.m_z) - (vertex1.m_z * vertex2.m_x)) +
                    vertex3.m_z * ((vertex1.m_y * vertex2.m_x) - (vertex1.m_x * vertex2.m_y));
            if (c >= 0.0f)
                ok = false;
        }

        return ok;
    }

    protected void removeAllVertexesClipped()
    {
        int i;

        for (i = 0; i < m_numVertexesClipped; i ++)
            m_listVertexesClipped[i] = null;
        m_numVertexesClipped = 0;
    }
    
    protected void addVertexClipped(GPJPoint3D vertex)
    {
        m_listVertexesClipped[m_numVertexesClipped ++] = vertex;
    }

    protected void addFirstVertexClipped()
    {
        m_listVertexesClipped[m_numVertexesClipped ++] = m_listVertexesClipped[0];
    }
    
}
