/*
 * GPJVector3D.java v1.2
 *
 * Copyright (C) 2010  by Giuseppe Perniola
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class GPJVector3D
{

    public float m_x;
    public float m_y;
    public float m_z;

    public GPJVector3D()
    {
    }

    public GPJVector3D(float x,
                       float y,
                       float z)
    {
        set(x,
                y,
                z);
    }

    public void set(float x,
                    float y,
                    float z)
    {
        m_x = x;
        m_y = y;
        m_z = z;
    }

    public static void copyVector(GPJVector3D src,
                                  GPJVector3D dst)
    {
        dst.m_x = src.m_x;
        dst.m_y = src.m_y;
        dst.m_z = src.m_z;
    }

    public float getMagnitude()
    {
        return (float)Math.sqrt(m_x * m_x +
                m_y * m_y +
                m_z * m_z);
    }

    public float getMagnitude2()
    {
        return m_x * m_x + m_y * m_y + m_z * m_z;
    }

    public static void addVectors(GPJVector3D src1,
                                  GPJVector3D src2,
                                  GPJVector3D dst)
    {
        dst.m_x = src1.m_x + src2.m_x;
        dst.m_y = src1.m_y + src2.m_y;
        dst.m_z = src1.m_z + src2.m_z;
    }

    public static void subVectors(GPJVector3D src1,
                                  GPJVector3D src2,
                                  GPJVector3D dst)
    {
        dst.m_x = src1.m_x - src2.m_x;
        dst.m_y = src1.m_y - src2.m_y;
        dst.m_z = src1.m_z - src2.m_z;
    }

    public static void reverseVector(GPJVector3D src,
                                     GPJVector3D dst)
    {
        dst.m_x = -src.m_x;
        dst.m_y = -src.m_y;
        dst.m_z = -src.m_z;
    }

    public static void multiplyVectorScalar(GPJVector3D src,
                                            float       value,
                                            GPJVector3D dst)
    {
        dst.m_x = src.m_x * value;
        dst.m_y = src.m_y * value;
        dst.m_z = src.m_z * value;
    }

    public static void normalizeVector(GPJVector3D src,
                                       GPJVector3D dst)
    {
        float magnitude;

        magnitude = src.getMagnitude();
        if (magnitude <= GPJEngine3D.TOLLERANCE)
            magnitude = 1.0f;

        dst.m_x = src.m_x / magnitude;
        dst.m_y = src.m_y / magnitude;
        dst.m_z = src.m_z / magnitude;
    }

    public static float dotProductVectors(GPJVector3D src1,
                                          GPJVector3D src2)
    {
        return src1.m_x * src2.m_x +
               src1.m_y * src2.m_y +
               src1.m_z * src2.m_z;
    }

    public static void crossProductVectors(GPJVector3D src1,
                                           GPJVector3D src2,
                                           GPJVector3D dst)
    {
        dst.m_x = src1.m_y * src2.m_z - src1.m_z * src2.m_y;
        dst.m_y = src1.m_z * src2.m_x - src1.m_x * src2.m_z;
        dst.m_z = src1.m_x * src2.m_y - src1.m_y * src2.m_x;
    }

    public static void multiplyVectorMatrix(GPJVector3D src1,
                                            GPJMatrix   src2,
                                            GPJVector3D dst)
    {
        dst.m_x = src1.m_x * src2.m_matrix[0] +
                src1.m_y * src2.m_matrix[1] +
                src1.m_z * src2.m_matrix[2];
        dst.m_y = src1.m_x * src2.m_matrix[3] +
                src1.m_y * src2.m_matrix[4] +
                src1.m_z * src2.m_matrix[5];
        dst.m_z = src1.m_x * src2.m_matrix[6] +
                src1.m_y * src2.m_matrix[7] +
                src1.m_z * src2.m_matrix[8];
    }

    @Override
    public String toString()
    {
        return super.toString() + "  x: " + m_x + "  y: " + m_y + "  z: " + m_z;
    }

}
